@@ -8,14 +8,17 @@ use Carp qw( cluck confess );
 use SQL::Abstract;
 use SQL::Abstract::Limit;
 use Class::DBI::Lite::Iterator;
+use Class::DBI::Lite::Pager;
 use Class::DBI::Lite::RootMeta;
 use Class::DBI::Lite::EntityMeta;
+use Digest::MD5 'md5_hex';
+use POSIX 'ceil';
 use overload 
   '""'      => sub { eval { $_[0]->id } },
   bool      => sub { eval { $_[0]->id } },
   fallback  => 1;
 
-our $VERSION = '1.001';
+our $VERSION = '1.003';
 our $meta;
 
 our %DBI_OPTIONS = (
@@ -87,7 +90,7 @@ sub construct
   my ($s, $data) = @_;
   
   my $class = ref($s) ? ref($s) : $s;
-  
+
   my $PK = $class->primary_column;
   my $key = join ':', grep { defined($_) } ( $s->root_meta->{schema}, $class, $data->{ $PK } );
   return $Live_Objects{$key} if $Live_Objects{$key};
@@ -125,6 +128,8 @@ sub dsn    { $_[0]->root_meta->{dsn} }
 sub table  { $_[0]->_meta->{table} }
 sub triggers { @{ $_[0]->_meta->{triggers}->{ $_[1] } } }
 sub _meta { }
+sub set_cache { my ($class, $cache) = @_; $class->_meta->{cache} = $cache }
+sub cache { shift->_meta->{cache} }
 
 
 #==============================================================================
@@ -282,9 +287,16 @@ sub retrieve
 {
   my ($s, $id) = @_;
   
+  if( my $data = $s->_call_triggers( before_retrieve => $id ) )
+  {
+    return $s->construct( $data );
+  }# end if()
+  
   my ($obj) = $s->retrieve_from_sql(<<"", $id);
     @{[ $s->primary_column ]} = ?
 
+  return unless $obj;
+  $obj->_call_triggers( after_retrieve => $obj );
   return $obj;
 }# end retrieve()
 
@@ -317,7 +329,7 @@ sub create
   $data = { %$pre_obj  };
   
   my @fields  = map { $_ } sort grep { exists($data->{$_}) } keys(%create_fields);
-  my @vals    = map { $data->{$_} } sort grep { exists($data->{$_}) } keys(%create_fields);
+  my @vals    = map { $data->{$_} } sort grep { exists($pre_obj->{$_}) } keys(%create_fields);
   
   my $sql = <<"";
     INSERT INTO @{[ $s->table ]} (
@@ -333,10 +345,15 @@ sub create
     or confess "ERROR - CANNOT get last insert id";
   $sth->finish();
   
-  $pre_obj->{$PK} = $id;
-  $pre_obj->_call_triggers( after_create => $pre_obj );
-  $pre_obj->update if $pre_obj->{__Changed};
-  $pre_obj;
+  my $new_obj = $s->construct( {
+    %$pre_obj,
+    $PK => $id,
+  } );
+  $pre_obj->discard_changes;
+
+  $new_obj->_call_triggers( after_create => $new_obj );
+  $new_obj->update if $new_obj->{__Changed};
+  $new_obj;
 }# end create()
 
 
@@ -515,17 +532,44 @@ sub sth_to_objects
 sub search
 {
   my ($s, %args) = @_;
+
+  my @cached = grep { $_ } $s->_call_triggers( before_search => \%args );
+  if( @cached )
+  {
+    return $s->_prepare_result( @cached );
+  }# end if()
   
   my $sql = "";
-
   my @sql_parts = map { "$_ = ?" } sort keys(%args);
   my @sql_vals  = map { $args{$_} } sort keys(%args);
   $sql .= join ' AND ', @sql_parts;
-  
-  return $s->retrieve_from_sql( $sql, @sql_vals );
+
+  my @vals = $s->retrieve_from_sql( $sql, @sql_vals );
+  $s->_call_triggers( after_search => ( \%args, \@vals ) );
+  return $s->_prepare_result( @vals );
 }# end search()
 
 
+sub _prepare_result
+{
+  my ($class, @vals) = @_;
+  if( wantarray )
+  {
+    my @out = map { $class->construct( $_ ) } @vals;
+    return @out;
+  }
+  else
+  {
+    my $iter = Class::DBI::Lite::Iterator->new(
+      [
+        map { $class->construct( $_ ) } @vals
+      ]
+    );
+    return $iter;
+  }# end if()
+}# end _prepare_result()
+
+
 #==============================================================================
 sub count_search
 {
@@ -604,23 +648,114 @@ sub search_where
 }# end search_where()
 
 
+#==============================================================================
+sub pager
+{
+  my ($s, $where, $attr) = @_;
+  
+  unless( $where && keys %$where )
+  {
+    $where = { 1 => 1 };
+  }# end unless()
+
+  foreach(qw( page_size page_number ))
+  {
+    confess "Required attribute '$_' was not provided"
+      unless $attr->{$_};
+  }# end foreach()
+  
+  # Limits:
+  my $page_size = $attr->{page_size};
+  my $page_number = $attr->{page_number};
+  my $offset = $page_number == 1 ? 0 : ($page_number - 1) * $page_size;
+
+  my $order = $attr ? $attr->{order_by} : undef;
+  my $sql = SQL::Abstract::Limit->new(%$attr, limit_dialect => $s->db_Main );
+  my($phrase, @bind) = $sql->where($where, $order);
+  $phrase =~ s/^\s*WHERE\s*//i;
+  
+  my $total = $s->count_search_where( $where );
+  my $total_pages = $total < $page_size ? 1 : POSIX::ceil($total / $page_size);
+  
+  return Class::DBI::Lite::Pager->new(
+    where       => $where,
+    order_by    => $order,
+    class       => ref($s) ? ref($s) : $s,
+    page_number => $page_number,
+    page_size   => $page_size,
+    total_pages => $total_pages,
+    total_items => $total,
+    start_item  => $offset + 1,
+    stop_item   => $offset + $page_size,
+  );
+}# end pager()
+
+
+#==============================================================================
+sub sql_pager
+{
+  my ($s, $args, $attr) = @_;
+  
+  confess "\$args is required" unless $args;
+  foreach( qw( data_sql count_sql ) )
+  {
+    confess "\$args->{$_} is required" unless $args->{$_};
+  }# end foreach()
+  $args->{sql_args} ||= [ ];
+
+  foreach(qw( page_size page_number ))
+  {
+    confess "Required attribute '$_' was not provided"
+      unless $attr->{$_};
+  }# end foreach()
+  
+  # Limits:
+  my $page_size = $attr->{page_size};
+  my $page_number = $attr->{page_number};
+  my $offset = $page_number == 1 ? 0 : ($page_number - 1) * $page_size;
+
+  # Get the total items:
+  my $sth = $s->db_Main->prepare( $args->{count_sql} );
+  $sth->execute( @{ $args->{sql_args} } );
+  my ($total) = $sth->fetchrow;
+  $sth->finish;
+  
+  my $total_pages = $total < $page_size ? 1 : POSIX::ceil($total / $page_size);
+  
+  return Class::DBI::Lite::Pager->new(
+    data_sql    => $args->{data_sql},
+    count_sql   => $args->{count_sql},
+    sql_args    => $args->{sql_args},
+    class       => ref($s) ? ref($s) : $s,
+    page_number => $page_number,
+    page_size   => $page_size,
+    total_pages => $total_pages,
+    total_items => $total,
+    start_item  => $offset + 1,
+    stop_item   => $offset + $page_size,
+  );
+}# end sql_pager()
+
+
 #==============================================================================
 sub count_search_where
 {
   my $s = shift;
   
   my $where = (ref $_[0]) ? $_[0]          : { @_ };
-  my $attr  = (ref $_[0]) ? $_[1]          : undef;
-  my $order = ($attr)     ? delete($attr->{order_by}) : undef;
-  my $limit  = ($attr)    ? delete($attr->{limit})    : undef;
-  my $offset = ($attr)    ? delete($attr->{offset})   : undef;
-  
-  my $abstract = SQL::Abstract::Limit->new(%$attr, limit_dialect => $s->db_Main );
-  my($phrase, @bind) = $abstract->where($where, $order, $limit, $offset);
-  $phrase =~ s/^\s*WHERE\s*//i;
-  
-  my $sql = "SELECT COUNT(*) FROM @{[ $s->table ]} WHERE $phrase";
+  my $phrase = "";
+  my @bind;
+  if( keys( %$where ) == 1 && (keys %$where)[0] eq '1' && (values %$where)[0] eq '1' )
+  {
+    # No phrase:
+  }
+  else
+  {
+    my $abstract = SQL::Abstract::Limit->new();
+    ( $phrase, @bind ) = $abstract->where($where);
+  }# end if()
   
+  my $sql = "SELECT COUNT(*) FROM @{[ $s->table ]} $phrase";
   SCOPE: {
     my $sth = $s->db_Main->prepare_cached($sql);
     $sth->execute( @bind );
@@ -637,10 +772,12 @@ sub find_or_create
 {
   my ($s, %args) = @_;
   
-  my ($obj) = $s->search( %args );
-  return $obj if $obj;
+  if( my ($obj) = $s->search( %args ) )
+  {
+    return $obj;
+  }# end if()
   
-  $s->create( %args );
+  return $s->create( %args );
 }# end find_or_create()
 
 
@@ -718,17 +855,32 @@ sub add_trigger
 sub _call_triggers
 {
   my ($s, $event) = @_;
-  
+
   $s->_meta->{triggers}->{ $event } ||= [ ];
   return unless my @handlers = @{ $s->_meta->{triggers}->{ $event } };
   shift;shift;
+  my @return_values;
+  my $return_value;
   foreach my $handler ( @handlers )
   {
-    eval {
-      $handler->( $s, @_ );
-      1;
-    } or confess $@;
+    if( wantarray )
+    {
+      eval {
+        my @val = $handler->( $s, @_ );
+        push @return_values, @val if @val;
+        1;
+      } or confess $@;
+    }
+    else
+    {
+      eval {
+        $return_value = $handler->( $s, @_ );
+        1;
+      } or confess $@;
+    }# end if()
   }# end foreach()
+  
+  return wantarray ? @return_values : $return_value;
 }# end _call_triggers()
 
 
@@ -745,11 +897,36 @@ sub dbi_commit
 sub remove_from_object_index
 {
   my $s = shift;
-  my $obj = delete($Live_Objects{$s->root_meta->{schema} . ':' . ref($s) . ':' . $s->id });
+  my $obj = delete($Live_Objects{ $s->get_cache_key });
   undef(%$obj);
 }# end remove_from_object_index()
 
 
+sub get_cache_key
+{
+  my $s = shift;
+  if( my $id = shift )
+  {
+    my $class = ref($s) ? ref($s) : $s;
+    return join ':', ( $s->root_meta->{schema}, $class, $id );
+  }
+  else
+  {
+    return $s->root_meta->{schema} . ':' . ref($s) . ':' . $s->id
+  }# end if()
+}# end get_cache_key()
+
+
+sub as_hashref
+{
+  my $s = shift;
+  my %data = %$s;
+  delete( $data{__Changed} );
+  delete( $data{__id} );
+  \%data;
+}# end as_hashref()
+
+
 #==============================================================================
 sub dbi_rollback
 {
@@ -1477,6 +1654,53 @@ Here is the example:
     ...
   }
 
+=head2 pager( \%where, { order_by => 'fields ASC', page_number => 1, page_size => 10 } )
+
+Returns a L<Class::DBI::Lite::Pager> object.
+
+Example:
+
+  # Step 1: Get our pager:
+  my $pager = My::Artist->pager({
+    name => { LIKE => 'Bob%' }
+  }, {
+    order_by    => 'name ASC',
+    page_number => 1,
+    page_size   => 20,
+  });
+  
+  # Step 2: Show the items in that recordset:
+  foreach my $artist ( $pager->items ) {
+    # Do stuff with $artist:
+    print $artist->name;
+  }
+
+See L<Class::DBI::Lite::Pager> for more details and examples.
+
+=head2 sql_pager( { data_sql => $str, count_sql => $str, sql_args => \@array }, { page_number => 1, page_size => 10 } )
+
+Returns a L<Class::DBI::Lite::Pager> object.
+
+Example:
+
+  # Step 1: Get our pager:
+  my $pager = My::Artist->sql_pager({
+    data_sql  => "SELECT * FROM artists WHERE name LIKE ?",
+    count_sql => "SELECT COUNT(*) FROM artists WHERE name LIKE ?",
+    sql_args  => [ 'Bob%' ],
+  }, {
+    page_number => 1,
+    page_size   => 20,
+  });
+  
+  # Step 2: Show the items in that recordset:
+  foreach my $artist ( $pager->items ) {
+    # Do stuff with $artist:
+    print $artist->name;
+  }
+
+See L<Class::DBI::Lite::Pager> for more details and examples.
+
 =head1 OBJECT METHODS
 
 =head2 Field Methods
@@ -1550,7 +1774,7 @@ Example:
 =head2 delete()
 
 Deletes the object from the database.  The object is then re-blessed into the special
-class C<Class::DBI::Object::That::Has::Been::Deleted>.
+class C<Class::DBI::Lite::Object::Has::Been::Deleted>.
 
 Example:
 
